/*******************************************************}
{                                                       }
{       File: demuxer.cpp                               }
{       Created by Tsviatko Jongov                      }
{       http://tsviatko.jongov.com                      }
{       Date : 25.01.2007                               }
{                                                       }
{       CMpeg2Demux class.                              }
{                                                       }
{*******************************************************/
#include "stdafx.h"
#include "demuxer.h"

static _inline unsigned int asm_swap32(unsigned int x)
{
	__asm 
	{
		mov eax, x;
		bswap eax;
	}
}

static _inline unsigned short asm_swap16(unsigned short x) 
{
	__asm 
	{
		mov ax, x;
		xchg al, ah;
	}
}

CMpeg2Demux::CMpeg2Demux()
{
	try
	{
		fBuffer = new CBuffer(1024 * 1024); 
		if (fBuffer == NULL)
		{
			printf("tsv: Error in CMpeg2Demux::CMpeg2Demux - can not create CBuffer.");  
		}
	}
	catch (...)
	{			
		printf("tsv: Error in CMpeg2Demux::CMpeg2Demux call...");  
	}
};

CMpeg2Demux::~CMpeg2Demux()
{
	try
	{
		if (fBuffer != NULL) {	
			delete fBuffer;
		}
	}
	catch (...)
	{		
		printf("tsv: Error in CMpeg2Demux::~CMpeg2Demux call...");  
	}
};

int CMpeg2Demux::AddData(void * Data, int Size)
{
	try
	{
		if ((Data == NULL) || (Size < 0))
		{		
			printf("tsv: Error in CMpeg2Demux::AddData - invalid data.");  
			return (-1);
		}

		fBuffer->add((char *)Data, Size);
		if (!Demultiplex())
		{	
			printf("tsv: Error in CMpeg2Demux::AddData - error demultiplexing.");  
			return (-1);
		}

		return (0);	
	}
	catch (...)
	{			
		printf("tsv: Error in CMpeg2Demux::AddData call...");  
		return (-1);
	}
}

void CMpeg2Demux::SetReceiveFunction(ReceiveFunction * func)
{	 
	try
	{	
		fReceiveFunction = func;
	}
	catch (...)
	{		
		printf("tsv: Error in CMpeg2Demux::SetReceiveFunction call...");  
	}
}

int CMpeg2Demux::Demultiplex()
{	 
	try
	{
		unsigned char * PDW;
		unsigned char * PDW2;
		unsigned int Code;
		int Processed = 0;
		int StreamID;
		int PES_packet_length;
		int PES_header_data_length;
		int Val;

		if (fBuffer->len() <= 0)
		{
			return (0);
		}
		 
		PDW = (unsigned char *)fBuffer->head(); 	 	
		while (fBuffer->len() - Processed > 140) 
		{	
			if ((*(unsigned int *)PDW & 0x00FFFFFF) == 0x00010000) 
			{ 
				Code = asm_swap32(*(unsigned int *)PDW);
				StreamID = (Code & 0xFF);

				if ((Code >= SYSTEM_START_CODE_MIN) &&
					(Code <= SYSTEM_START_CODE_MAX) &&           
					(Code != PACK_START_CODE) &&           
					(Code != SYSTEM_START_CODE) &&           
					(StreamID != PROGRAM_STREAM_MAP) &&           
					(StreamID != PADDING_STREAM) &&           
					(StreamID != PRIVATE_STREAM_2) &&           
					(StreamID != ECM_STREAM) &&           
					(StreamID != EMM_STREAM) &&           
					(StreamID != PROGRAM_STREAM_DIRECTORY) &&           
					(StreamID != DSM_CC_STREAM) &&           
					(StreamID != ITU_T_STREAM_E)) 
				{						
					PES_packet_length = asm_swap16(*(unsigned short *)(PDW + 4));
					PES_header_data_length = *(PDW + 8);
					
					if (fBuffer->len() - Processed >= PES_packet_length + 6 + 4)
					{
						PDW2 = (unsigned char *)(PDW + PES_packet_length + 6);            
						if ((*(unsigned int *)PDW2 & 0x00FFFFFF) != 0x00010000) 
						{
							printf("tsv: Warning in CMpeg2Demux::Demultiplex - ((*PDW & 0x00FFFFFF) != 0x00010000).");
							return (-1);
						}

						Val = 6 + 3 + PES_header_data_length;

						if (fReceiveFunction != NULL)
						{
							fReceiveFunction(StreamID, PDW + Val, PES_packet_length + 6 - Val);
						}

						fBuffer->fFirst += Processed + PES_packet_length + 6;
						fBuffer->compact();

						Processed = 0;           				
						PDW = (unsigned char *)fBuffer->head();
					}
					else
					{					
						break;
					}
				}
				else {								
					PDW++;
					Processed++;
				}
			}	 
			else {
				PDW++;
				Processed++;
			}
		};

		return (true);
	}
	catch (...)
	{
		printf("tsv: Error in CMpeg2Demux::Demultiplex call...");
		return (-1);
	}
}